// Written by Dr. YIP Chi Lap [Beta]
//   for Talk on "DIY Earthquake Detector: The software side" 
//   Earthquake Detector Design Competition
//   Faculty of Engineering, The University of Hong Kong, 2016.

import processing.video.*;

Movie mov;
int frameNumber;
int[] prevFrame;
long totalDiff;

// This function is called when the program first starts.
//
// A complication of using the Movie class:
//   there is no way to find the frame rate, so have to calculate.
void setup() {
  frameNumber = 0;

  // Creates the object representing the movie.
  mov = new Movie(this, "Circle trace.mov");
  size(728, 454, P2D);
  
  // mov = new Movie(this, "LongitudinalWave.mov");
  // size(1275, 119, P2D);
  
  // We have to start it playing and read the first frame
  //   so we can get properties of the movie, such as its size.
  mov.play();                      // Start video playing.
  while (!mov.available()) {       // Wait till video is available.
    print(".");			   // Print something while waiting.
    flush();			     // Make sure it is shown immediately.
  }
  mov.read();                      // Read the content of the frame.
  // Create the space to remember the previous frame.
  prevFrame = new int[mov.width*mov.height];
  // Tell the system that we are going to manipulate the display pixels.
  loadPixels();
}

// This function is called automatically whenever the screen can be refreshed.
void draw() {
  // Make background black.
  // This is not needed unless the movie is not completely opaque.
  background(0);
  // Read a new frame if it is available.
  if (mov.available()) {
    // Read the video frame and make the data available.
    mov.read();
    ++frameNumber;
    mov.loadPixels();
    // totalDiff will become the sum of brightness value differences
    //   for all the pixels in this frame.
    totalDiff = 0;
    
    for (int r=0; r<mov.width*mov.height; ++r) {
      int currBrightness = round(brightness(mov.pixels[r]));
      int prevBrightness = round(brightness(prevFrame[r]));
      // Absolute value is needed so we always get a positive number.
      int brightnessDiff = abs(currBrightness-prevBrightness);

      // Put the brightness difference for displaying.
      pixels[r] = color(brightnessDiff);

      // Remember the pixels of this frame for use next time
      prevFrame[r] = mov.pixels[r];
    }
    updatePixels();
  }
  // Draw a frame of movie on screen.
  // image(mov, 0, 0, width, height);
  // Make words to be displayed in dark green.
  // The three values are values for
  //   Red, Green, and Blue components respectively.
  fill(0,127,0);
  // Display the frame number on console and screen.
  println("Frame: "+frameNumber+" Diff:"+totalDiff);
  textSize(24);
  text("Frame: "+frameNumber+" Diff:"+totalDiff, 15, 30);

}